#include <stdio.h>
#include <stdlib.h>
#include <limits.h>

#include "bmp_util.h"

#define COLOR_BYTES (3)
#define COLOR_BITS  (24)

static int put_bmp_header(FILE *s, int x, int y, int c);
static int put_uint16(unsigned short d, FILE *s);
static int put_uint32(unsigned int d, FILE *s);

int
write_bmp(const char *name, unsigned char *buf, int width, int height)
{
  FILE *f;
  int r;

  if((f = fopen(name, "wb")) == NULL){
    return -1;
  }

  if((r = put_bmp_header(f, width, height, COLOR_BITS)) == 0){
    fclose(f);
    return -1;
  }

  int size = width * COLOR_BYTES * height;
  r = fwrite(buf, sizeof(unsigned char), size, f);
  if(r != size){
    fclose(f);
    return -1;
  }

  fclose(f);
  return 0;
}

int
put_bmp_header(FILE *s, int x, int y, int c)
{
  int i;
  int color;
  unsigned int bfOffBits;

  if(x <= 0 || y <= 0){
    return 0;
  }

  if(s == NULL || ferror(s)){
    return 0;
  }

  if(c == 24){
    color = 0;
  }else{
    color = 1;
    for( i = 1 ; i <= c ; i++){
      color *= 2;
    }
  }

  // header size = bitmap_file_header(14) + bitmap_info_header(40) + #. of colors
  bfOffBits = 14 + 40 + 4 * color;

  // bitmap_file_header
  fputs("BM", s); // identifier
  // bfSize filesize(byte)
  put_uint32(bfOffBits + x * y, s);
  // bfReserved1 reserved 1(byte)
  put_uint16(0, s);
  // bfReserved2 reserved 2(byte)
  put_uint16(0, s);
  // bfOffBits header size(byte)
  put_uint32(bfOffBits, s);

  // bitmap_info_header (40bytes)
  // biSize header size
  put_uint32(40, s);
  // biWidth image width
  put_uint32(x, s);
  // biHeight image height
  put_uint32(y, s);
  // biPlanes
  put_uint16(1, s);
  // biBitCount colors bit
  put_uint16(c, s);
  // biCompression
  put_uint32(0, s);
  // biSizeImage
  put_uint32(0, s);
  // biXPelsPerMeter
  put_uint32(0, s);
  // biYPelsPerMeter
  put_uint32(0, s);
  // biClrUsed
  put_uint32(0, s);
  // biClrImportant
  put_uint32(0, s);

  if(ferror(s)){
    return 0;
  }

  return 1;
}

int
put_uint16(unsigned short d, FILE *s)
{
  putc(d & 0xFF, s);
  return putc(d >> 8, s);
}

int
put_uint32(unsigned int d, FILE *s)
{
  putc(d & 0xFF, s);
  putc((d >> 8) & 0xFF, s);
  putc((d >> 16) & 0xFF, s);
  return putc((d >> 24) & 0xFF, s);
}
